#!/usr/bin/python
# -*- coding:utf-8 -*-

#  Copyright © 2011, 2013  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, division

import sys, os
from urlparse import urlparse

from gi.repository import GLib

import launchpadlib
import launchpadlib.credentials
from launchpadlib.credentials import AuthorizeRequestTokenWithBrowser, EndUserDeclinedAuthorization
from launchpadlib.launchpad import Launchpad
from launchpadlib.errors import HTTPError
from launchpadlib.uris import lookup_service_root
import webbrowser

CONSUMER_NAME = 'lp-browser'
APPLICATION_NAME = 'lp-browser'
CACHE_DIR = os.path.expanduser('~/.cache/launchpad/')

class AuthorizeRequestTokenWithBrowserGTK_1_9 (AuthorizeRequestTokenWithBrowser):
    """The simplest request token authorizer for GTK.

    This authorizer simply opens up the end-user's web browser to a
    Launchpad URL and lets the end-user authorize the request token
    themselves.
    """
    
    parent = None
    
    def make_end_user_authorize_token(self, credentials, request_token):
        """Have the end-user authorize the token in their browser."""
        authorization_url = self.authorization_url(request_token)
        webbrowser.open(authorization_url)
        
        dialog = Gtk.Dialog('Launchpad Authorization', self.parent,
                            Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT)
        label_message = Gtk.Label(label=self.WAITING_FOR_USER % authorization_url)
        label_message.show()
        dialog.vbox.pack_start(label_message, True, True, 0)
        label_error = Gtk.Label(label='')
        label_error.show()
        dialog.vbox.pack_start(label_error, True, True, 0)
        dialog.result = None
        
        def timer(dialog):
            try:
                credentials.exchange_request_token_for_access_token(self.web_root)
            except HTTPError, e:
                dialog.result = str(e)
                if e.response.status == 403:
                    # The user decided not to authorize this
                    # application.
                    #raise EndUserDeclinedAuthorization(e.content)
                    dialog.response(Gtk.ResponseType.NO)
                    return False
                elif e.response.status == 401:
                    # The user has not made a decision yet.
                    label_error.set_text('')
                    return True
                else:
                    # There was an error accessing the server.
                    label_error.set_text("Unexpected response from Launchpad:\n%s" % e)
                    return True
            else:
                dialog.response(Gtk.ResponseType.YES)
                return False
                    
        GLib.timeout_add_seconds(launchpadlib.credentials.access_token_poll_time, timer, dialog)
        result = dialog.run()
        dialog.destroy()
        if result != Gtk.ResponseType.YES:
            raise EndUserDeclinedAuthorization(dialog.result)
        
        
def login(service_root, window=None):
    '''Convenience function to login'''
    
    try:
        service_root_ = lookup_service_root(service_root)
    except ValueError:
        service_root_ = service_root
    service_loc = urlparse(service_root_).netloc or service_root
    cache_dir = os.path.join(CACHE_DIR, service_loc)
    credentials_file = os.path.join(cache_dir, 'credentials', CONSUMER_NAME)
    app_cache = os.path.join(cache_dir, CONSUMER_NAME+'.cache')
    
    def _login(anonymous=False):
        lp = None
        try:
            if anonymous:
                lp = Launchpad.login_anonymously(
                    CONSUMER_NAME,
                    service_root=service_root,
                    launchpadlib_dir=CACHE_DIR)
            else:
                AuthorizeRequestTokenWithBrowserGTK_1_9.parent = window
                lp = Launchpad.login_with(
                    APPLICATION_NAME,
                    service_root=service_root,
                    launchpadlib_dir=CACHE_DIR,
                    authorization_engine=AuthorizeRequestTokenWithBrowserGTK_1_9(
                                                service_root,
                                                APPLICATION_NAME),
                    )
                str(lp.me.name)
        except HTTPError as e:
            if e.response.status == '401':
                retry = 1
            elif e.response.status == '403':
                retry = 2
            else:
                retry = 0
            return None, retry, str(e)
        except EndUserDeclinedAuthorization as e:
            return None, 2, str(e)
        except Exception as e:
            return None, 0, str(e)
        return lp, 0, 'Login successful'
        
    lp, retry, error = _login()
    if retry == 1:
        os.remove(credentials_file)
        lp, retry, error = _login()
    if retry == 0:
        return lp, app_cache, error
    lp, retry, error = _login(anonymous=True)
    return lp, app_cache, error
    
