#!/usr/bin/python
# -*- coding:utf-8 -*-

#  Copyright © 2011, 2013  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, division

from gi.repository import Gtk

from lp_browser.helpers import get_builder


class OperationDialog(object):
    def __init__(self, parent=None):
        builder = get_builder('operation.ui')
        self.dialog = builder.get_object('operation_dialog')
        self.dialog.set_transient_for(parent)
        builder.get_object('button_ok').connect('clicked', self.on_ok)

        self.table_operation = builder.get_object('table_operation')
        self.operation = None
        self.operation_name = None
        self.params_entry = None

    def set_operation(self, attrname, operation, method_info):
        self.operation = operation
        operation_id = operation.wadl_method.id
        self.operation_name = operation.wadl_method.name
        assert self.operation_name in ['get', 'post']
        method_name, descr_short, descr_long, params_wadl = method_info[operation_id]
        assert method_name.lower() == self.operation_name
        assert params_wadl[0][0] == u'ws.op'

        from gi.repository import Pango
        def create_label(text, x, r, y, b, wrap=False, tooltip_text=None):
            label = Gtk.Label(label=text)
            label.props.xalign = 0. if x == 0 else 1.
            label.props.wrap = wrap
            if wrap:
                pal = Pango.AttrList()
                pal.insert(Pango.AttrStyle(Pango.Style.ITALIC, end_index=len(text)))
                label.props.attributes = pal
            label.props.tooltip_text = tooltip_text
            label.show()
            self.table_operation.attach(label, x, r, y, b)
        create_label(attrname, 0, 1, 0, 1)
        create_label(method_name, 1, 2, 0, 1)
        create_label(descr_short+'\n'+descr_long, 0, 2, 1, 2, wrap=True)

        self.params_entry = []
        for i, (key, required, name, description) in enumerate(params_wadl[1:]):
            if not name:
                name = key
            if required:
                name += ' *'
            create_label(name, 0, 1, i+2, i+3, tooltip_text=description)
            entry = Gtk.Entry()
            self.params_entry.append((key, entry))
            entry.show()
            self.table_operation.attach(entry, 1, 2, i+2, i+3)
        return True

    def on_ok(self, unused_widget):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.RESONSE_OK from run().
        """
        if self.operation_name == 'get':
            kwargs = {}
            for key, entry in self.params_entry:
                value = entry.get_text()
                if value:
                    kwargs[key] = value
            self.kwargs = kwargs
        else:
            dialog = Gtk.MessageDialog(self,
                            Gtk.DialogFlags.MODAL,
                            Gtk.MessageType.ERROR,
                            Gtk.ButtonsType.OK,
                            '"%s"-Operations not supported (yet).' % self.operation_name)
            dialog.run()
            dialog.destroy()


if __name__ == "__main__":
    dialog = OperationDialog()
    dialog.dialog.show()
    Gtk.main()
