#!/usr/bin/python
# -*- coding:utf-8 -*-

#  Copyright © 2011, 2013  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, division

from urlparse import urlparse

from gi.repository import Gtk
from launchpadlib.uris import web_roots, service_roots

from lp_browser.helpers import get_builder


class SelectserviceDialog(object):
    def __init__(self, parent=None):
        builder = get_builder('selectservice.ui')
        self.dialog = builder.get_object('selectservice_dialog')
        self.dialog.set_transient_for(parent)
        builder.get_object('button_ok').connect('clicked', self.on_ok)
        builder.get_object('radiobutton_staging').connect('toggled', self.on_radiobutton_staging_toggled)
        builder.get_object('radiobutton_entry').connect('toggled', self.on_radiobutton_entry_toggled)
        builder.get_object('radiobutton_production').connect('toggled', self.on_radiobutton_production_toggled)
        
        self.entry_service = builder.get_object('entry_service')
        self.label_error = builder.get_object('label_error')
        self._service = 'production'
        self.service = None

    def on_ok(self, unused_widget):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.RESONSE_OK from run().
        """
        self.service = self.entry_service.props.text if self._service is None else self._service

    def on_radiobutton_production_toggled(self, widget):
        if widget.props.active:
            self.entry_service.props.sensitive = False
            self._service = 'production'

    def on_radiobutton_staging_toggled(self, widget):
        if widget.props.active:
            self.entry_service.props.sensitive = False
            self._service = 'staging'

    def on_radiobutton_entry_toggled(self, widget):
        if widget.props.active:
            self.entry_service.props.sensitive = True
            self.entry_service.grab_focus()
            self._service = None

def validate_service(service):
    services = {}
    for k, v in service_roots.items():
        services[k] = k
        services[v] = k
        services[urlparse(v).netloc] = k
    for k, v in web_roots.items():
        services[v] = k
        services[urlparse(v).netloc] = k
    try:
        return services[service]
    except KeyError:
        url = urlparse(service)
        if url.scheme and (url.netloc or url.path):
            if url.netloc.startswith('api.'):
                return service
            else:
                return url._replace(netloc='api.'+url.netloc).geturl()
        else:
            return None

def select_service(parent=None):
    dialog = SelectserviceDialog(parent)
    while True:
        result = dialog.dialog.run()
        if result != Gtk.ResponseType.OK:
            service = None
            break
        service = validate_service(dialog.service)
        if service is not None:
            break
        dialog.label_error.props.visible = True
    dialog.dialog.destroy()
    return service

if __name__ == "__main__":
    print('service: ' + (select_service() or 'none'))

